Package: graphonmix
Type: Package
Title: Generates Dense and Sparse Graphs using Graphon Extensions
Version: 0.1.1
Authors@R: c(
   person("Sevvandi", "Kandanaarachchi", email = "sevvandik@gmail.com",
  role = c("aut", "cre"), comment = c(ORCID = "0000-0002-0337-0395"))
    )
Maintainer: Sevvandi Kandanaarachchi <sevvandik@gmail.com>
Description: Generates dense or sparse graphs using graphon mixtures and graphettes. Graphon mixtures uses two graphons U and W to generate graphs. Sparse graphs are generated in this case using the inverse line graph (root) operation. Graphettes have 3 components, the graphon W, a real-valued sequence and a graph edit function. Both techniques can generate dense or sparse graphs. Kandanaarachchi and Ong (2026) <doi:10.48550/arXiv.2505.13864>, Wijesinghe et al (2026) <doi:10.48550/arXiv.2602.23566>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: ggplot2, igraph, imager, stats
Suggests: gridExtra, knitr, rmarkdown
VignetteBuilder: knitr
URL: https://sevvandi.github.io/graphonmix/
NeedsCompilation: no
Packaged: 2026-06-12 06:20:37 UTC; kan092
Author: Sevvandi Kandanaarachchi [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0337-0395>)
Repository: CRAN
Date/Publication: 2026-06-12 07:50:02 UTC
Built: R 4.6.0; ; 2026-06-12 11:03:40 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(autoplot,extract_sparse)
S3method(autoplot,separate_dense_and_sparse)
export(autoplot)
export(create_exp_matrix)
export(empirical_graphon)
export(extract_sparse)
export(generate_star_union)
export(graph_join)
export(line_graphon)
export(plot_graphon)
export(predict_hubs)
export(ring_graphon)
export(sample_graphette)
export(sample_graphon)
export(sample_mixed_graph)
export(sample_sparse_graphon)
export(sbm_graphon)
export(scale_graphon)
export(separate_dense_and_sparse)
export(star_f1)
export(star_f2)
export(star_f3)
export(star_f4)
export(star_f5)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(igraph,"%du%")
importFrom(igraph,degree)
importFrom(igraph,ecount)
importFrom(igraph,vcount)
importFrom(stats,coef)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,predict)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# graphonmix <a href="https://sevvandi.github.io/graphonmix/"><img src="man/figures/logo.png" align="right" height="138" alt="graphonmix website" /></a>

<!-- badges: start -->
[![R-CMD-check](https://github.com/sevvandi/graphonmix/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/sevvandi/graphonmix/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The goal of graphonmix is to generate dense or sparse graphs via
different techniques. We explore $(U,W)$-mixture graphs and graphettes.
$(U,W)$-mixture graphs are generated from a standard dense graphon $W$
and a disjoint clique graphon $U$, which acts as the sparse graph
generator. Graphettes are a triple consisting of a graphon $W$, a
sparsifing sequence $\{\rho_n\}_n$ with $\rho_n \to 0$ and graph edit
function $f$. $(U,W)$-mixture graphs are discussed in (Kandanaarachchi
and Ong 2026) and graphettes are discussed in (Wijesinghe et al. 2026).

## Installation

You can install the development version of graphonmix from
[GitHub](https://github.com/) with:

``` r
# install.packages("pak")
pak::pak("sevvandi/graphonmix")
```

## $(U,W)$ Mixture Example

This is a basic example on how to sample a $(U,W)$-mixture graph.

``` r
library(graphonmix)
library(igraph)
#> 
#> Attaching package: 'igraph'
#> The following objects are masked from 'package:stats':
#> 
#>     decompose, spectrum
#> The following object is masked from 'package:base':
#> 
#>     union
# create the dense graphon W(x,y) = 0.1
W <- matrix(0.1, nrow = 100, ncol = 100)
# create the sparse part - a disjoint set of stars
wts <- c(0.5, 0.3, 0.2)
# single function to generate a graph mixture
gr1 <- sample_mixed_graph(W, wts, nd = 100, ns = 300, p = 0.5, option = 2)
plot(gr1,
     edge.curved = 0.3,
     vertex.size = degree(gr1)*0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "(U,W) Graph mixture"
)
```

<img src="man/figures/README-example-1.png" width="100%" />

<!-- Or you can generate the two graphs separately and join them.  -->
<!-- ```{r} -->
<!-- # sample the dense part and plot -->
<!-- grdense <- sample_graphon(W, 100) -->
<!-- plot(grdense, -->
<!--      edge.curved = 0.3, -->
<!--      vertex.size = degree(grdense)*0.1, -->
<!--      edge.color = "lightgray",     # Light colored edges -->
<!--      vertex.label = NA, -->
<!--      vertex.color = "lightblue", -->
<!--      main = "Dense Part" -->
<!-- ) -->
<!-- # sample the sparse part and plot -->
<!-- grsparse <- generate_star_union(wts, 300) -->
<!-- plot(grsparse, -->
<!--      edge.curved = 0.3, -->
<!--      vertex.size = degree(grsparse)*0.1, -->
<!--      edge.color = "lightgray",     # Light colored edges -->
<!--      vertex.label = NA, -->
<!--      vertex.color = "lightblue", -->
<!--      main = "Sparse Part" -->
<!-- ) -->
<!-- # join the two graphs and plot -->
<!-- gr2 <- graph_join(grdense, grsparse, option = 2) -->
<!-- plot(gr2, -->
<!--      edge.curved = 0.3, -->
<!--      vertex.size = degree(gr2)*0.1, -->
<!--      edge.color = "lightgray",     # Light colored edges -->
<!--      vertex.label = NA, -->
<!--      vertex.color = "lightblue", -->
<!--      main = "(U,W) Graph mixture" -->
<!-- ) -->
<!-- ``` -->

## Graphette example

This is an example with $\rho_n = 10/n$ and $f = \text{star_f1}$. The
graph edit function $\text{star_f1}$ adds stars using a Poisson process.

``` r
rho <- function(n) 20/n
gr <- sample_graphette(W, 
                       rho_n = rho, 
                       graph_edit_f = 'star_f1',
                       n = 200,
                       t_or_p = 2)

plot(gr, vertex.label = NA, vertex.size = 3)
```

<img src="man/figures/README-unnamed-chunk-2-1.png" width="100%" />

## Acknowledgements

A big thank you to Sashenka Fernando for helping me with the hex
sticker.

## References

<div id="refs" class="references csl-bib-body hanging-indent"
entry-spacing="0">

<div id="ref-SKCS2025graphon" class="csl-entry">

Kandanaarachchi, Sevvandi, and Cheng Soon Ong. 2026. “Graphon Mixtures.”
*AISTATS*.

</div>

<div id="ref-wijesinghe2026flowette" class="csl-entry">

Wijesinghe, Asiri, Sevvandi Kandanaarachchi, Daniel M Steinberg, and
Cheng Soon Ong. 2026. “Flowette: Flow Matching with Graphette Priors for
Graph Generation.” *arXiv Preprint arXiv:2602.23566*.

</div>

</div>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(igraph)
library(graphonmix)
library(ggplot2)

## -----------------------------------------------------------------------------
# create the dense graphon W(x,y) = exp(-(x+y)/40) where x and y ranges from 1 to 100
W <- create_exp_matrix(100, 40)
# plot this graphon
plot_graphon(W) +
 coord_fixed(ratio = 1) +
 ggtitle("Dense graphon W")  

## -----------------------------------------------------------------------------
# weights for the sparse part
seq <- 2:5
wts <- (1/1.2^seq) 
wts <- wts/sum(wts)
wts

U <- line_graphon(wts)
plot_graphon(U) + 
  coord_fixed(ratio = 1) +
 ggtitle("Line graphon U")  


## -----------------------------------------------------------------------------

# single function to generate a graph mixture
gr1 <- sample_mixed_graph(W, wts, nd = 100, ns = 300, p = 0.5)
# plot(gr1, vertex.label = NA, vertex.size = 1, main = "(U,W) Graph mixture")
plot(gr1,
     edge.curved = 0.3,
     vertex.size = degree(gr1)*0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "(U,W) Graph mixture"
)

## -----------------------------------------------------------------------------
# sample dense part and plot it
grdense <- sample_graphon(W, n = 100)
plot(grdense,
     edge.curved = 0.3,
     vertex.size = degree(grdense) * 0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "Dense Part"
)

# sample sparse part and plot it
grsparse <- generate_star_union(wts, n = 300)
plot(grsparse,
     edge.curved = 0.3,
     vertex.size = degree(grsparse) * 0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "Sparse Part"
)

# join the two graphs and plot it
gr2 <- graph_join(grdense, grsparse, p = 0.5)
plot(gr2,
     edge.curved = 0.3,
     vertex.size = degree(gr2) * 0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "(U,W) Graph mixture"
)

## -----------------------------------------------------------------------------
degu <- sort(unique(degree(gr2)), decreasing = TRUE)
# we only take the top 1/2 of the unique degree values here to see the effect of the hub nodes clearly
degu <- degu[degu >= median(degu)]

df <- data.frame(
  index = 1:length(degu),
  log_degree = log(degu)
)

ggplot(df, aes(index, log_degree)) + 
 geom_point() + 
 xlab("Index") + 
 ylab("Log Degree")  



## -----------------------------------------------------------------------------
deg_dense <- sort(unique(degree(grdense)), decreasing = TRUE)
deg_dense

deg_sparse <- sort(unique(degree(grsparse)), decreasing = TRUE)
deg_sparse

deg_mix <- sort(unique(degree(gr2)), decreasing = TRUE)
deg_mix

## -----------------------------------------------------------------------------
which(deg_sparse > max(deg_dense))

## -----------------------------------------------------------------------------
out <- extract_sparse(gr2)
out$num_hubs

# Estimate of the mass-partition
out$phat

# Actual mass partition
wts

## -----------------------------------------------------------------------------
autoplot(out)

## -----------------------------------------------------------------------------
Uhat <- line_graphon(out$phat)
plot_graphon(Uhat) + 
  coord_fixed(ratio = 1) +
 ggtitle("Estimate of U")  

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(graphonmix)
library(ggplot2)
library(igraph)
requireNamespace("gridExtra", quietly = TRUE)

## -----------------------------------------------------------------------------
g1 <- make_star(4,mode = "undirected")
g1 <- add_vertices(g1, 1)
g1 <- add_edges(g1,c(4,5))

lg1 <- make_line_graph(g1)

oldpar <- par(mfrow = c(1,2))
plot(g1, vertex.label = NA, edge.label = c(1,2,3,4), main = "Graph G")
plot(lg1, vertex.label = c(1,2,3,4), vertex.size = 20, main = "Line graph L(G)")
par(oldpar)

## -----------------------------------------------------------------------------
st <- make_star(10, mode = "undirected")
lgr_st <- make_line_graph(st)

oldpar <- par(mfrow = c(1,2))
plot(st, vertex.label = NA, main = "Star graph G")
plot(lgr_st, vertex.label = NA,  main = "Line graph L(G) of a star")
par(oldpar)

## -----------------------------------------------------------------------------
stardensity <- gnpdensity <- rep(0, 20)
 
for(i in 1:20){
  n <- i*100
  gr1 <- sample_gnp(n, p = 0.1)
  gnpdensity[i] <- edge_density(gr1)
  gr2 <- make_star(n, mode = "undirected")
  stardensity[i] <- edge_density(gr2)
}
gnpdensity
stardensity

## -----------------------------------------------------------------------------
gr <- make_star(10, mode = "undirected")
emp <- empirical_graphon(gr)
plot_graphon(emp) + coord_fixed()

## -----------------------------------------------------------------------------
gr <- make_star(20, mode = "undirected")
emp <- empirical_graphon(gr)
g1 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 10')

gr <- make_star(100, mode = "undirected") 
emp <- empirical_graphon(gr)
g2 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 100')

gr <- make_star(200, mode = "undirected")
emp <- empirical_graphon(gr)
g3 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 200')


gridExtra::grid.arrange(g1, g2, g3, nrow = 1)


## -----------------------------------------------------------------------------
gr1 <- make_star(10, mode = "undirected")
gr <- gr1 %du% gr1
lgr <- make_line_graph(gr)
emp <- empirical_graphon(lgr)
g1 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 20')


gr1 <- make_star(50, mode = "undirected")
gr <- gr1 %du% gr1
lgr <- make_line_graph(gr)
emp <- empirical_graphon(lgr)
g2 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 100')


gr1 <- make_star(100, mode = "undirected")
gr <- gr1 %du% gr1
lgr <- make_line_graph(gr)
emp <- empirical_graphon(lgr)
g3 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 200')


gridExtra::grid.arrange(g1, g2, g3, nrow = 1)



## -----------------------------------------------------------------------------
wts <- c(0.5, 0.3, 0.2)
U <-  line_graphon(wts)
gr <- sample_graphon(U, n = 100)
plot(gr,
     vertex.size = 1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue"
)

## -----------------------------------------------------------------------------
grsp <- generate_star_union(wts, n = 100)
plot(grsp,
     edge.curved = 0.3,
     vertex.size = 1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue"
)

#' Samples a graph from a sparsified graphon
#'
#' Samples a graph from a sparsified graphon given by a_n W, where W is the graphon and a_n is generally a sequence converging to zero.
#'
#' @param W The graphon. This is a symmetric matrix with values in [0,1]
#' @param rho_n A function. Default is 1.
#' @param n The number of nodes in the graph to sample
#'
#' @return gr A graph sampled from rho_n W.
#'
#' @examples
#' library(igraph)
#' f1 <- function(n) 10/n
#' W <- matrix(0.2, nrow = 100, ncol = 100)
#' gr <- sample_sparse_graphon(W, f1, n = 100)
#' gr
#'
#' @export
sample_sparse_graphon <- function(W, rho_n = NULL, n){

  # rho_n is a function

  # check if W is a symmatrix matrix
  if (!is.matrix(W)) {
    stop("Input W is not a matrix.")
  }

  if (nrow(W) != ncol(W)) {
    stop("W is not square.")
  }

  if (!isSymmetric(W)) {
    stop("W is not symmetric.")
  }

  if( (max(W) >= 1) |(min(W) <= 0)){
    stop("W contains values outside of [0,1].")
  }

  if(is.null(rho_n)){
    rho_n  <- function(n) 1
  }else if(rho_n(10) < rho_n(100)){
    stop("Function rho_n cannot be increasing!")
  }

  Wn <- rho_n(n)*W
  gr <- sample_graphon(Wn, n)
  gr
}



#' Samples a graph from a graphette
#'
#' Samples a graph from a graphette given by (W, a_n, f), where W is the graphon, a_n is generally a sequence converging to zero and f is a graph edit funcion.
#'
#' @param W The graphon. This is a symmetric matrix with values in [0,1]
#' @param rho_n A function. Default is 1.
#' @param graph_edit_f The graph edit function. For star functions it can be
#' one of \code{'star_f1'}, \code{'star_f2'}, \code{'star_f3'}, \code{'star_f4'} or \code{'star_f5'}. To add rings, it needs to be \code{'add_rings'}. The function \code{'remove_cycles'} removes cycles.
#' @param n The number of nodes in the graph to sample
#' @param t_or_p The parameter for star function or the ring function. For the
#' ring function it needs to be between 0 and 1.
#' @param ring_sizes The size of rings to add if the graph edit function is the ring function. default values set to \code{c(5,6)}.
#'
#' @return gr A graph sampled from rho_n W.
#'
#' @examples
#' library(igraph)
#' # Example 1
#' f1 <- function(n) 10/n
#' W <- matrix(0.2, nrow = 100, ncol = 100)
#' gr <- sample_graphette(W, rho_n = f1, graph_edit_f = 'add_rings', n = 100, t_or_p = 0.5)
#' gr
#'
#' # Example 2
#' gr <- sample_graphette(W, rho_n = f1, graph_edit_f = 'star_f1', n = 100, t_or_p = 3)
#' gr
#' @export
sample_graphette <- function(W,
                             rho_n = NULL,
                             graph_edit_f = NULL,
                             n = 100,
                             t_or_p = 0.5,
                             ring_sizes = c(5,6)){

  gr1 <- sample_sparse_graphon(W, rho_n, n)

  star_func_list <- list('star_f1', 'star_f2', 'star_f3', 'star_f4', 'star_f5')

  if(is.null(graph_edit_f)){
    gr <- gr1
  }else if(graph_edit_f %in% star_func_list){
    gr <- add_star_edges(gr1, match.fun(graph_edit_f), t_or_p)
  }else if (graph_edit_f == 'add_rings'){
    if(t_or_p > 1 | t_or_p < 0){
      stop('Parameter t_or_p need to be between 0 and 1 for rings!')
    }
    gr <- add_rings(gr1, proportion = t_or_p, ring_sizes = ring_sizes)
  }else if (graph_edit_f =='remove_cycles'){
    gr <- remove_cycles(gr1)
  }

  gr
}


add_star_edges <- function(gr, star_f, t) {
  # This function adds star edges to a graph based on a star graph function.
  #
  # Args:
  #   gr: An igraph graph object.
  #   star_f: A function representing the starness of nodes in a graph.
  #   t: Time
  #
  # Returns:
  #   A modified igraph graph object with added star edges.

  num_nodes <- vcount(gr)
  star_samples <- sample_star_function(star_f, num_nodes, t)

  star_inds <- which(star_samples > 0)
  for (i in star_inds) {
    num_star_edges <- star_samples[i]
    # add num_star_edges vertices to gr
    new_vertices <- seq(num_nodes + 1, num_nodes + num_star_edges)
    gr <- igraph::add_vertices(gr, nv = num_star_edges)
    # connect vertex i to new vertices
    gr <- igraph::add_edges(gr, c(rbind(rep(i, num_star_edges), new_vertices)))
    num_nodes <- igraph::vcount(gr)
  }

  return(gr)
}


#' Adds stars to nodes
#'
#' @param xvals Values between 0 and 1 from the graphon
#' @param t A parameter indicating the strength of hubs
#'
#' @return a vector of values
#'
#' @examples
#' # example code
#' st <- star_f1(runif(10), 3)
#' st
#' @export
star_f1 <- function(xvals, t){
  vals <- exp(-1*(xvals - 0.2)*t)
  return(vals)
}

#' Adds stars to nodes
#'
#'@inheritParams star_f1
#' @return a vector of values
#'
#' @examples
#' # example code
#' st <- star_f2(runif(10), 3)
#' st
#' @export
star_f2 <- function(xvals, t){
  vals <- t*exp(-1*(xvals - 0.2))
  return(vals)
}

#' Adds stars to nodes
#'
#'@inheritParams star_f1
#' @return a vector of values
#'
#' @examples
#' # example code
#' st <- star_f3(runif(10), 3)
#' st
#' @export
star_f3 <- function(xvals, t){
  vals <- t*exp(-1*(xvals + 1))
  return(vals)
}

#' Adds stars to nodes
#'
#'@inheritParams star_f1
#' @return a vector of values
#'
#' @examples
#' # example code
#' st <- star_f4(runif(10), 3)
#' st
#' @export
star_f4 <- function(xvals, t){
  k=3
  vals <- rep(k*t, length(xvals))
  return(vals)
}

#' Adds stars to nodes
#'
#'@inheritParams star_f1
#' @return a vector of values
#'
#' @examples
#' # example code
#' st <- star_f5(runif(10), 3)
#' st
#' @export
star_f5 <- function(xvals, t){
  vals <- t/10*exp(-1*(xvals - 0.2))
  return(vals)
}

# Star functions and edge addition for graphs
sample_star_function <- function(star_f, num_samples, t) {
  # This function samples from a star graph function.
  #
  # Args:
  #   star_f: A function representing the starness of nodes in a graph.
  #   num_samples: Number of samples to draw.
  #   t: Time
  #
  # Returns:
  #   A vector of sampled values from the star graph function.

  xvals <- seq(0, 1, length.out = num_samples)
  star_vals <- star_f(xvals = xvals, t = t)
  samples <- stats::rpois(num_samples, lambda = star_vals)

  return(samples)
}



add_rings <- function(gr, proportion = 0.1, ring_sizes = c(5, 6)) {
  # Input validation
  if (!igraph::is.igraph(gr)) {
    stop("Input must be an igraph object")
  }
  if (proportion <= 0 || proportion > 1) {
    stop("Proportion must be between 0 and 1")
  }

  # Calculate number of rings to add
  n_vertices <- vcount(gr)
  n_rings <- max(1, round(n_vertices * proportion))

  # Start with the original graph
  result_graph <- gr

  # Add rings
  for (i in 1:n_rings) {
    # Randomly choose ring size (5 or 6 vertices)
    ring_size <- sample(ring_sizes, 1)

    # Create a ring
    ring <- igraph::make_ring(ring_size)

    # Get current number of vertices before adding ring
    current_n <- igraph::vcount(result_graph)

    # Combine the graphs (disjoint union)
    result_graph <- igraph::disjoint_union(result_graph, ring)

    # Select a random node from the original graph to connect to
    selected_node <- sample(1:current_n, 1)

    # Select a random node from the newly added ring to connect from
    # The ring vertices are now indexed from (current_n + 1) to (current_n + ring_size)
    ring_node <- sample((current_n + 1):(current_n + ring_size), 1)

    # Add an edge connecting the selected node to the ring
    result_graph <- igraph::add_edges(result_graph, c(selected_node, ring_node))
  }

  return(result_graph)
}


remove_cycles <- function(gr) {
  # Convert to a directed acyclic graph by removing back edges
  # Use a spanning tree approach
  gr <- igraph::as.undirected(gr)
  spanning_tree <- igraph::mst(gr)
  return(spanning_tree)
}

#' Plots the output of separate_dense_and_sparse.
#'
#' Plots the three lines fitted to the unique, sorted log degrees of the graph.
#'
#' @param object The output of the function 'separate_dense_and_sparse.'
#' @param ... Other arguments currently ignored
#'
#' @return A ggplot object.
#'
#' @examples
#' library(igraph)
#' W <- matrix(0.1, nrow = 100, ncol = 100)
#' wts <- c(0.5, 0.3, 0.2)
#' ns <- 200
#' nd <- 100
#' p <- 0.5
#' gr <- sample_mixed_graph(W, wts, nd, ns, p = 0.1, option = 2)
#' out <- separate_dense_and_sparse(gr)
#' separate <- separate_dense_and_sparse(gr)
#' autoplot(separate)
#'
#' @export
autoplot.separate_dense_and_sparse <- function(object, ...) {
  x <- y <- segment <- NULL

  # Extract data and cutoffs
  result <- object$fitlines
  df <- result$data
  i <- result$cutoffs[1]
  j <- result$cutoffs[2]

  # Create data for the fitted lines
  x_range1 <- seq(1, i, length.out = 100)
  x_range2 <- seq(i+1, j, length.out = 100)
  x_range3 <- seq(j+1, nrow(df), length.out = 100)

  # Predict values for each segment
  y_pred1 <- predict(result$models[[1]], newdata = data.frame(x = x_range1))
  y_pred2 <- predict(result$models[[2]], newdata = data.frame(x = x_range2))
  y_pred3 <- predict(result$models[[3]], newdata = data.frame(x = x_range3))

  # Create data frames for the fitted lines
  line_df1 <- data.frame(x = x_range1, y = y_pred1, segment = "Segment 1")
  line_df2 <- data.frame(x = x_range2, y = y_pred2, segment = "Segment 2")
  line_df3 <- data.frame(x = x_range3, y = y_pred3, segment = "Segment 3")

  # Combine all line data
  line_df <- rbind(line_df1, line_df2, line_df3)

  # Create data for elbow points
  elbow_df <- data.frame(
    x = c(i, j),
    y = df$y[c(i, j)]
  )

  # Create the plot
  p <- ggplot(df, aes(x = x, y = y)) +
    geom_point(size = 2) +
    geom_line(data = line_df, aes(x = x, y = y, color = segment), size = 1) +
    geom_point(data = elbow_df, aes(x = x, y = y), color = "purple", size = 3, shape = 19) +
    ggplot2::theme_minimal() +
    ggplot2::scale_color_manual(
      values = c("Segment 1" = "red", "Segment 2" = "blue", "Segment 3" = "green"),
      name = "Lines"
    ) +
    ggplot2::annotate(
      "text",
      x = c(i, j),
      y = df$y[c(i, j)],
      label = c("Elbow 1", "Elbow 2"),
      vjust = -1,
      hjust = 0.5
    ) +
    ggplot2::labs(x = "Index", y = "Log Degree")

  return(p)
}

#' Plots the output of extract_sparse function.
#'
#' Plots the two lines fitted to the unique, sorted log degrees of the graph.
#'
#' @param object The output of the function 'extract_sparse'
#' @param ... Other arguments currently ignored
#'
#' @return A ggplot object.
#'
#' @examples
#' library(igraph)
#' gr <- sample_pa(10000, power = 1.2, directed = FALSE)
#' sparse <- extract_sparse(gr)
#' autoplot(sparse)
#'
#' @export
autoplot.extract_sparse <- function(object, ...){
  x <- y <- segment <- NULL

  # Extract data and num_hubs
  df <- object$data
  i <- object$cutoff

  # Create data for the fitted lines
  x_range1 <- seq(1, i, length.out = 100)
  x_range2 <- seq(i+1, nrow(df), length.out = 100)

  # Predict values for each segment
  y_pred1 <- predict(object$models[[1]], newdata = data.frame(x = x_range1))
  y_pred2 <- predict(object$models[[2]], newdata = data.frame(x = x_range2))

  # Create data frames for the fitted lines
  line_df1 <- data.frame(x = x_range1, y = y_pred1, segment = "Sparse")
  line_df2 <- data.frame(x = x_range2, y = y_pred2, segment = "Dense")

  # Combine all line data
  line_df <- rbind(line_df1, line_df2)

  # Create data for elbow point
  elbow_df <- data.frame(
    x = i,
    y = df$y[i]
  )

  # Produce plot
  p <- ggplot(df, aes(x = x, y = y)) +
    geom_point(size = 1) +
    ggplot2::geom_line(data = line_df, aes(x = x, y = y, color = segment), size = 1) +
    ggplot2::theme_bw() +
    xlab("Index") +
    ylab("Log Degree")
  p
}

arrange_matrix <- function(mat){
  rowsums <- apply(mat, 1, sum)
  ord <- order(rowsums, decreasing = TRUE)
  mat2 <- mat[ord, ]
  colsums <- apply(mat, 2, sum)
  ord <- order(colsums, decreasing = TRUE)
  mat3 <- mat2[ ,ord]
  mat3
}

#' Plots graphon
#'
#' Plots graphon
#'
#' @param W A graphon given by an nxn matrix
#' @param cols Colors, by default white and black
#'
#' @return A ggplot object.
#'
#' @examples
#' W <- create_exp_matrix(100, 100)
#' plot_graphon(W)
#'
#' @export
plot_graphon <- function(W, cols=c("white","black")){
  column <- val <- NULL
  adj1 <- W
  df1 <- data.frame(val = c(matrix(adj1)),
                    row = rep(rep(1:dim(adj1)[1],each = dim(adj1)[1]),
                              len = length(adj1)),
                    column = rep(1:dim(adj1)[1], len = length(adj1)))
  g2 <- ggplot(df1, aes(x = row, y = column, fill = val)) +
    ggplot2::geom_raster() +
    ggplot2::scale_y_reverse() +
    ggplot2::geom_rect(xmin = min(df1$row)-0.5 , xmax = max(df1$row)+0.5 ,
              ymin = -1*max(df1$column)-0.5 , ymax = -1*min(df1$column)+0.5 ,
              fill = NA, color = "black", size = 0.5, inherit.aes = FALSE) +
    ggplot2::scale_fill_gradientn(colours = cols,
                                  limits = c(0,1)) +
    ggplot2::theme_void() +
    ggplot2::theme(legend.position = "none")

  g2
}

#' Creates an exponential graphon
#'
#' Creates an nxn matrix where the (i,j)th entry is exp(-(i+j)/scalar)
#'
#' @param nrow The dimension of the matrix
#' @param scalar The scalar in exp(-(i+j)/scalar)
#'
#' @return An nxn matrix
#'
#' @examples
#' W <- create_exp_matrix(100, 100)
#'
#' @export
create_exp_matrix <- function(nrow, scalar) {
  # Initialize an empty matrix of the desired dimensions
  result <- matrix(0, nrow = nrow, ncol = nrow)

  # Fill each (i,j) position with exp(-i-j)
  for (i in 1:nrow) {
    for (j in 1:nrow) {
      result[i, j] <- exp(-(i+j)/scalar)
    }
  }

  return(result)
}


#' Creates a ring graphon
#'
#' Creates an graphon which can generate ring graphs.
#'
#' @param n The dimension of the matrix
#' @param alpha A scaling factor used in the graphon. Default set to 0.1
#'
#' @return An nxn matrix
#'
#' @examples
#' # example code
#' W <- ring_graphon(100)
#' plot_graphon(W)
#' @export
ring_graphon <- function(n, alpha = 0.1){
  mat <- matrix(0, nrow = n, ncol = n)
  for(i in 1:n){
    x <- i/n
    y <- (1:n)/n
    mat[i, ] <- 0.9*exp((-y^2 - (x-1)^2)/alpha^2) + 0.9*exp((-(y-1)^2 - x^2)/alpha^2) + 0.9*exp(-((sin(0.75*pi)*x + cos(0.75*pi)*y )/alpha)^2)
  }
  mat
}


#' Creates a Stochastic Block Model graphon
#'
#' Creates an graphon representing a Stochastic Block Model (SBM).
#'
#' @param mat The matrix representing the SBM
#' @param n The dimension of the matrix
#'
#' @return An nxn matrix
#'
#' @examples
#' # example code
#' mat <- matrix(c(0.9, 0.01, 0.02,
#' 0.01, 0.8, 0.03,
#' 0.02, 0.03, 0.7), nrow = 3, byrow = TRUE)
#' W <- sbm_graphon(mat, 100)
#' plot_graphon(W)
#' @export
sbm_graphon <- function(mat, n){
  k <- nrow(mat)
  block_size <- n / k
  graphon <- matrix(0, nrow = n, ncol = n)

  for(i in 1:k){
    for(j in 1:k){
      row_start <- floor((i - 1) * block_size) + 1
      row_end <- floor(i * block_size)
      col_start <- floor((j - 1) * block_size) + 1
      col_end <- floor(j * block_size)

      graphon[row_start:row_end, col_start:col_end] <- mat[i, j]
    }
  }

  return(graphon)
}

#' #' Computes the Hill estimator or the power of the degree distribution
#' #'
#' #' Given a graph, computes the Hill estimator or the power of the
#' #' degree distribution.
#' #'
#' #' @param gr A graph
#' #' @param min_deg The minimum degree to consider. Default set to 5
#' #'
#' #' @return The Hill estimate
#' #'
#' #' @examples
#' #' library(igraph)
#' #' gr <- sample_pa(10000,  directed = FALSE)
#' #' hill_estimator(gr)
#' #'
#' #'
#' hill_estimator <- function(gr, min_deg = 2){
#'   sorted_degs <-  sort(igraph::degree(gr))
#'   unique_degs <- unique(sorted_degs)
#'   inds <- which(sorted_degs >= min_deg)
#'   tail_degs <- sorted_degs[inds]
#'
#'   df <- as.data.frame(table(tail_degs))
#'
#'   dmin <- (min(tail_degs)-1)
#'   j <- sort(unique(tail_degs))
#'   d_j <- df$Freq
#'
#'   numerator <- sum(d_j*log(j/dmin))
#'   # or
#'   # numerator <- sum(tail_degs*log(inds/min(tail_degs)))
#'
#'   denominator <- sum(d_j)
#'   numerator/denominator
#' }

#' Extracts the sparse part from a (U,W) graphon mixture
#'
#' This function extracts the sparse component from a (U,W) mixture graph by
#' fitting two lines to the unique sorted log degree values. The first line gives
#' the number of hubs in the sparse part and the remaining line fits the degrees
#' of the dense part.
#'
#' @param gr The input graph
#'
#' @return a list with the following components:
#' \item{\code{num_hubs}}{The number of hubs in the sparse component.}
#' \item{\code{phat}}{The probability vector of the sparse component. This is also known as the mass partition.}
#' \item{\code{segment_sizes}}{The sizes of the two line segments.}
#' \item{\code{line_equations}}{The two equations of the lines.}
#' \item{\code{cutoff}}{The best cut off for the two lines.}
#' \item{\code{models}}{The models of the fitted lines. }
#' \item{\code{mse}}{The mean squared error}
#' \item{\code{data}}{The degree data.}
#'
#' @examples
#' library(igraph)
#' gr <- sample_pa(10000, power = 1.2, directed = FALSE)
#' sparse <- extract_sparse(gr)
#' sparse$phat
#'
#'@export extract_sparse
extract_sparse <- function(gr){
  # gr is a graph
  degs_nonuniq <- sort(igraph::degree(gr), decreasing = TRUE)
  degs <- sort(unique(igraph::degree(gr)), decreasing = TRUE)
  log_degs <- log(degs + 1)
  log_degs <- log_degs[log_degs > median(log_degs)]
  out <- fit_two_segments(log_degs)
  kk <- out$num_hubs
  kknew <- sum(degs_nonuniq >= degs[kk])  # Adjust num_hubs to be the number of points above the elbow point
  out$num_hubs <- kknew

  # Get the mass-partition (sparse component probabilities)
  degree_sum <- sum(sort(degree(gr), decreasing = TRUE)[1:kknew])
  phat <- sort(degree(gr), decreasing = TRUE)[1:kknew]/degree_sum

  out$phat <- phat
  out
}


fit_two_segments <- function(y) {
  # Create x values from 1 to length(y)
  x <- 1:length(y)
  n <- length(y)

  # Create a data frame with the points
  df <- data.frame(x = x, y = y)

  # Initialize variables to store best fit
  best_mse <- Inf
  best_i <- 0
  best_models <- list()

  # Try all possible changepoints
  for (i in 2:(n-1)) {
    # Segment 1: points 1 to i
    segment1 <- lm(y ~ x, data = df[1:i, ])

    # Segment 2: points (i+1) to n
    segment2 <- lm(y ~ x, data = df[(i+1):n, ])

    # Calculate mean squared error for this combination
    pred1 <- predict(segment1, newdata = df[1:i, ])
    pred2 <- predict(segment2, newdata = df[(i+1):n, ])

    mse1 <- mean((df$y[1:i] - pred1)^2)
    mse2 <- mean((df$y[(i+1):n] - pred2)^2)

    # Weighted average MSE based on segment sizes
    mse <- (i * mse1 + (n - i) * mse2) / n

    # Update if this is the best fit so far
    if (mse < best_mse) {
      best_mse <- mse
      best_i <- i
      best_models <- list(segment1, segment2)
    }
  }

  # Extract coefficients from the best models
  coef1 <- coef(best_models[[1]])
  coef2 <- coef(best_models[[2]])

  # Format the line equations
  line1_eq <- sprintf("y = %.4f * x + %.4f", coef1[2], coef1[1])
  line2_eq <- sprintf("y = %.4f * x + %.4f", coef2[2], coef2[1])


  # Return results
  structure(list(
    num_hubs = best_i,
    phat = NULL,
    cutoff = best_i,
    segment_sizes = c(best_i, n - best_i),
    line_equations = c(line1_eq, line2_eq),
    models = best_models,
    mse = best_mse,
    data = df,
    call = match.call()
  ), class = 'extract_sparse')


}

#' Separates the dense and sparse part from a (U,W) graphon mixture
#'
#' This function breaks a (U,W) mixture graph into a dense and sparse component.
#'
#' @param grmix The input graph
#'
#' @return a list with the following components:
#' \item{\code{gr_dense}}{The dense component.}
#' \item{\code{gr_sparse}}{The sparse component.}
#' \item{\code{data}}{The original graph.}
#'
#' @examples
#' library(igraph)
#' W <- matrix(0.1, nrow = 100, ncol = 100)
#' wts <- c(0.5, 0.3, 0.2)
#' ns <- 200
#' nd <- 100
#' p <- 0.5
#' gr <- sample_mixed_graph(W, wts, nd, ns, p = 0.1, option = 2)
#' out <- separate_dense_and_sparse(gr)
#' out
#'
#'@export separate_dense_and_sparse
separate_dense_and_sparse <- function(grmix){
  out <- extract_sparse_and_dense(grmix)
  num_hubs <- out$num_hubs
  sorted_stars <- sort(degree(grmix), decreasing = TRUE)[1:num_hubs]
  for(i in 1:num_hubs){
    st <- igraph::make_star(n = sorted_stars[i], mode = "undirected")
    star_vertex <- which(degree(grmix) == sorted_stars[i] )
    if(i == 1){
      gr_sparse <- st
    }else{
      gr_sparse <- gr_sparse %du% st
    }
  }

  # Remove isolated edges
  # both endpoints have degree == 1
  gr_dense <- out$dense_part
  ends <- igraph::ends(gr_dense, igraph::E(gr_dense))  # matrix of edge endpoints

  isolated_edges <- igraph::E(gr_dense)[degree(gr_dense)[ends[,1]] == 1 &
                                          degree(gr_dense)[ends[,2]] == 1]

  gr_dense <- igraph::delete_edges(gr_dense, isolated_edges)

  # Delete isolated vertices from gr_dense
  gr_dense <- igraph::delete.vertices(gr_dense, which(degree(gr_dense) == 0))


  # Return results
  structure(list(
    dense_part = gr_dense,
    sparse_part = gr_sparse,
    data = grmix,
    fitlines = out,
    call = match.call()
  ), class = 'separate_dense_and_sparse')
}


extract_sparse_and_dense <- function(gr){
  # gr is a graph
  degs_nonuniq <- sort(igraph::degree(gr), decreasing = TRUE)
  degs <- sort(unique(igraph::degree(gr)), decreasing = TRUE)
  log_degs <- log(degs + 2)
  #log_degs <- log_degs[log_degs > median(log_degs)]
  out <- fit_three_segments(log_degs)
  kk <- out$num_hubs
  kknew <- sum(degs_nonuniq >= degs[kk])  # Adjust num_hubs to be the number of points above the elbow point
  out$num_hubs <- kknew

  # Get the mass-partition (sparse component probabilities)
  degree_sum <- sum(sort(degree(gr), decreasing = TRUE)[1:kknew])
  phat <- sort(degree(gr), decreasing = TRUE)[1:kknew]/degree_sum
  out$phat <- phat

  # dense part
  degs_dense <- degs_nonuniq[(degs_nonuniq < degs[kk]) & (degs_nonuniq >= degs[out$dense_limit])]
  dense_nodes <- which(degree(gr) %in% degs_dense)
  dense_part <- igraph::subgraph(gr, dense_nodes)
  out$dense_part <- dense_part
  out
}



fit_three_segments <- function(y) {
  # Create x values from 1 to length(y)
  x <- 1:length(y)
  n <- length(y)

  # Create a data frame with the points
  df <- data.frame(x = x, y = y)

  # Initialize variables to store best fit
  best_mse <- Inf
  best_i <- 0
  best_j <- 0
  best_models <- list()

  # Try all possible combinations of changepoints
  for (i in 2:(n-2)) {
    for (j in (i+1):(n-1)) {
      # Segment 1: points 1 to i
      segment1 <- lm(y ~ x, data = df[1:i, ])

      # Segment 2: points (i+1) to j
      segment2 <- lm(y ~ x, data = df[(i+1):j, ])

      # Segment 3: points (j+1) to n
      segment3 <- lm(y ~ x, data = df[(j+1):n, ])

      # Calculate mean squared error for this combination
      pred1 <- predict(segment1, newdata = df[1:i, ])
      pred2 <- predict(segment2, newdata = df[(i+1):j, ])
      pred3 <- predict(segment3, newdata = df[(j+1):n, ])

      mse1 <- mean((df$y[1:i] - pred1)^2)
      mse2 <- mean((df$y[(i+1):j] - pred2)^2)
      mse3 <- mean((df$y[(j+1):n] - pred3)^2)

      # Weighted average MSE based on segment sizes
      mse <- (i * mse1 + (j - i) * mse2 + (n - j) * mse3) / n

      # Update if this is the best fit so far
      if (mse < best_mse) {
        best_mse <- mse
        best_i <- i
        best_j <- j
        best_models <- list(segment1, segment2, segment3)
      }
    }
  }

  # Extract coefficients from the best models
  coef1 <- coef(best_models[[1]])
  coef2 <- coef(best_models[[2]])
  coef3 <- coef(best_models[[3]])

  # Format the line equations
  line1_eq <- sprintf("y = %.4f * x + %.4f", coef1[2], coef1[1])
  line2_eq <- sprintf("y = %.4f * x + %.4f", coef2[2], coef2[1])
  line3_eq <- sprintf("y = %.4f * x + %.4f", coef3[2], coef3[1])

  # Return results
  result <- list(
    num_hubs = best_i,
    dense_limit = best_j,
    cutoffs = c(best_i, best_j),
    segment_sizes = c(best_i, best_j - best_i, n - best_j),
    elbow_points = c(best_i, best_j),
    line_equations = c(line1_eq, line2_eq, line3_eq),
    models = best_models,
    mse = best_mse,
    data = df
  )

  return(result)
}

#' Predicts the degree of hubs of an new graph
#'
#' Predicts the degree of hubs of an unseen graph given a graph generated
#' from the same process.
#'
#' @param gr The input graph
#' @param n The number of nodes in the new graph
#' @param k The number of hubs. Default is \code{NULL}
#'
#' @return A vector of hub degrees
#'
#' @examples
#' library(igraph)
#' gr <- sample_pa(10000, power = 1.2, directed = FALSE)
#' predict_hubs(gr, n = 11000)
#' @export
predict_hubs <- function(gr, n, k = NULL){
  # gr is the current graph
  # n is the number of nodes of the new graph
  # k is the number of hubs
  if(is.null(k)){
    sparse <- extract_sparse(gr)
    k <- sparse$num_hubs
  }
  current_hub_degs <- sort(igraph::degree(gr), decreasing = TRUE)[1:k]
  num_nodes <- igraph::vcount(gr)
  new_hub_degs <- round(current_hub_degs/num_nodes*n)
  new_hub_degs

}

#' @importFrom ggplot2 ggplot aes geom_raster xlab ylab geom_point geom_line scale_y_reverse
#' @importFrom ggplot2 autoplot scale_fill_gradientn theme theme_bw element_blank
#' @importFrom stats coef lm median predict
#' @importFrom igraph %du% ecount degree vcount
#' @export
ggplot2::autoplot
NULL

#' @docType package
#' @aliases NULL graphonmix-package
#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

#' Generates a sparse graph of star graphs
#'
#' Generates a union of star graphs given the weights
#'
#' @param wts The proportion of the hub degrees
#' @param n The number of nodes in the new graph
#'
#' @return A disjoint union of star graphs
#'
#' @examples
#' library(igraph)
#' wts <- c(0.5, 0.3, 0.2)
#' gr <- generate_star_union(wts, n = 100)
#' gr
#'
#' @export
generate_star_union <- function(wts, n){
  # wts is the mass partition
  # stopifnot("Weights should add up to 1" = sum(wts) == 1)
  if(sum(wts) != 1){
    wts <- wts/sum(wts)
    message("Rescaling weights to add to 1")
  }
  n = as.integer(n)

  star_sizes <- ceiling(n*wts)
  for(i in 1:length(star_sizes)){
    gr <- igraph::make_star(star_sizes[i], mode= "undirected")
    if(i == 1){
      gr2 <- gr
    }else{
      gr2 <- gr2 %du% gr
    }
  }
  gr2
}


#' Joins two graphs
#'
#' Joins two graphs randomly connecting vertices
#'
#' @param gr1 The first graph to join
#' @param gr2 The second graph to join
#' @param p The proportion of edges in \code{gr1} to be added as part of the joining
#' @param  option Two options. 1 does the disjoint union, 2 does the random edges union.
#'
#' @return The joined graph
#'
#' @examples
#' W <- create_exp_matrix(100, 100)
#' # create the sparse part - a disjoint set of stars
#' wts <- c(0.5, 0.3, 0.2)
#' grdense <- sample_graphon(W, 100)
#' grsparse <- generate_star_union(wts, 200)
#' gr <- graph_join(grdense, grsparse, opt = 2)
#'
#'@export
graph_join <- function(gr1, gr2, p = 0.5, option = 2) {
  # Check that inputs are igraph objects
  if (!igraph::is.igraph(gr1) || !igraph::is.igraph(gr2)) {
    stop("Both inputs must be igraph objects")
  }

  # Get the number of vertices in each graph
  n1 <- igraph::vcount(gr1)
  n2 <- igraph::vcount(gr2)

  # Create a disjoint union of the two graphs
  joined_graph <- igraph::disjoint_union(gr1, gr2)

  if(option == 2){
    # The other option - not the disjoint union
    edgecount <- ceiling(ecount(gr1)*p)
    node1_sample <- sample(1:(n1+n2), edgecount, replace = TRUE)
    node2_sample <- sample(1:(n1+n2), edgecount, replace = TRUE)
    new_edges <- cbind(node1_sample, node2_sample)
    # Add the new edges to the joined graph
    joined_graph <- igraph::add_edges(joined_graph, as.vector(t(as.matrix(new_edges))))
    joined_graph <- igraph::simplify(joined_graph)
  }

  # Preserve vertex names if they exist
  if (!is.null(igraph::V(gr1)$name) && !is.null(igraph::V(gr2)$name)) {
    igraph::V(joined_graph)$name <- c(igraph::V(gr1)$name, igraph::V(gr2)$name)
  }

  # Return the joined graph
  return(joined_graph)
}


#' Generate a (U,W) mixture graph
#'
#' Generate a (U,W) mixture graph from a dense graphon W and a mass partition
#' corresponding to a line graph graphon U.
#'
#' @param  W The dense graphon. This is a symmetric matrix with values in [0,1]
#' @param wts The degree proportions of the hub degrees. Need to add up to 1. This
#' is the mass partition corresponding to the line graph graphon U.
#' @param nd The number of nodes in the dense part of the graph
#' @param ns The number of nodes in the sparse part of the graph
#' @param p The nodes to be added as a proportion of the edges in the dense part
#' @param option Graph joining option. If \code{option == 1} then a disjoit union is
#' considered. If \code{option == 2} the two graphs are joined randomly with the
#' number of edges specified by \code{p}.
#'
#' @return A graph sampled from the (U,W) mixture.
#'
#' @examples
#' library(igraph)
#' W <- matrix(0.1, nrow = 100, ncol = 100)
#' wts <- c(0.5, 0.3, 0.2)
#' ns <- 200
#' nd <- 100
#' p <- 0.5
#' gr <- sample_mixed_graph(W, wts, nd, ns, p, option = 2)
#' gr
#' @export
sample_mixed_graph <- function(W, wts, nd, ns, p = 0.5, option = 2){
  # W is a graphon
  stopifnot("W is not a matrix " = is.matrix(W))
  stopifnot("W is not symmetric " = isSymmetric(W))
 #  stopifnot("wts need to add to 1 " = sum(wts) == 1)
  stopifnot("p needs to be between 0 and 1 " = (p >=0) & (p <= 1))
  stopifnot("Option needs to be either 1 or 2 " = option %in% c(1,2))

  if(sum(wts) != 1){
    wts <- wts/sum(wts)
    message("Rescaling weights to add to 1")
  }
  nd <- as.integer(nd)
  ns <- as.integer(ns)
  grdense <- sample_graphon(W, nd)
  grsparse <- generate_star_union(wts, ns)
  gr <- graph_join(grdense, grsparse, p=p,  option = option)
  gr
}



#' Generates a graph given a graphon
#'
#' Generates a graph given a dense graphon W.
#'
#' @param W A graphon given by a matrix
#' @param n The number of nodes of the sampled graph
#'
#' @return A graph sampled from the graphon W with n nodes
#'
#' @examples
#' library(igraph)
#' W <- matrix(0.2, nrow = 100, ncol = 100)
#' gr <- sample_graphon(W, n= 100)
#'
#' @export
sample_graphon <- function(W, n){
  graphonscaled <- scale_graphon(W, n)
  adj <- generate_adj_from_probs(graphonscaled)
  gr <- igraph::graph_from_adjacency_matrix(adj, mode = "undirected")
  gr
}


#' Computes empirical graphon from graph
#'
#' Computes empirical graphon given a graph
#'
#' @param gr A graph
#' @param n Dimension of the graphon matrix
#'
#' @return The empirical graphon
#'
#' @examples
#' library(igraph)
#' gr <- sample_gnp(1000, p=0.2)
#' emp <-  empirical_graphon(gr, n = 100)
#'
#' @export
empirical_graphon <- function(gr, n=NULL){
  adj <- igraph::get.adjacency(gr)
  adj2 <- arrange_matrix(adj)
  if(!is.null(n)){
    adj2 <- scale_graphon(adj2, n)
  }
  adj2
}


generate_adj_from_probs <- function(probsmat) {
  probs <- probsmat[upper.tri(probsmat, diag=FALSE)]
  vals <- stats::rbinom(n = length(probs), size = 1, prob = probs)
  adj <- matrix(0, nrow = nrow(probsmat), ncol = nrow(probsmat))
  adj[upper.tri(adj, diag=FALSE)] <- vals
  adj <- adj + t(adj)
  diag(adj) <- 0
  adj
}


#' Scales a graphon to an nxn matrix
#'
#' Scales a graphon to an nxn matrix suitable for large adjacency matrices
#'
#' @param W A graphon given as a symmetric square matrix
#' @param n The dimension of the output matrix
#'
#' @return Scaled nxn graphon
#'
#' @examples
#' library(igraph)
#' gr <- sample_gnp(1000, p=0.2)
#' adj <- as_adjacency_matrix(gr)
#' W <- scale_graphon(adj, 100)
#'
#' @export
scale_graphon <- function(W, n){
  # W is a n x n matrix
  # scalar is a scalar
  # n is the size of the output matrix
  W <- as.matrix(W)
  img <- imager::as.cimg(W)
  Wscaled <- as.matrix(imager::resize(img, n, n, interpolation_type = 2))
  Wscaled
}


#' Creates a line graphon from a sequence of probabilities
#'
#' Creates a line graphon, which is a disjoint clique graphon from a
#' sequence of probability values
#'
#' @param probs The list of probabilities starting from the largest.
#'
#' @return The line graphon
#'
#' @examples
#' library(ggplot2)
#' wts <- c(0.5, 0.3, 0.2)
#' U <- line_graphon(wts)
#' plot_graphon(U)
#'
#' @export
line_graphon <- function(probs){
  # Generates a line graphon from a list of probabilities

  if(sum(probs) == 1){
    minp <- min(probs)
    nn <- 3*length(probs)/minp
    len <- length(probs)
    st <- 1
    mat <- matrix(0, nrow = nn, ncol = nn)
    for(kk in 1:len){
      if(st < nn){
        prob_len <- round(probs[kk]*nn)
        en <- min(st + prob_len-1, nn)
        mat[st:en, st:en] <- 1
        st <- st + prob_len
      }
    }
  }else{
    # sum probs < 1
    minp <- min(probs)
    zeroprob <- 1 - sum(probs)
    probs <- c(probs, zeroprob)
    nn <- 3*length(probs)/minp
    len <- length(probs)
    st <- 1
    mat <- matrix(0, nrow = nn, ncol = nn)
    for(kk in 1:(len-1)){
      if(st < nn){
        prob_len <- round(probs[kk]*nn)
        en <- min(st + prob_len -1, nn)
        mat[st:en, st:en] <- 1
        st <- st + prob_len
      }
    }
  }
  mat
}

---
title: "Graphon Mixtures"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Graphon Mixtures}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


```{r setup}
library(igraph)
library(graphonmix)
library(ggplot2)
```

# (U,W)-mixture graphs

The $(U,W)$-mixture graphs are graphs generated from two graphons $U$ and $W$. Graphon $W$ generates dense graphs and graphon $U$ generates sparse graphs. The vignette at *Articles/Sparse Graphs from Line Graphons* explains how graphon $U$ generates sparse graphs. Once the dense and sparse graphs are generated, nodes are joined randomly. This gives the mixture graph. The methodology is explained in [@SKCS2025graphon].

<!-- As a consequence of the Aldous-Hoover theorem graphons produce dense graphs (to be technically correct, dense or empty graphs). So how do we generate sparse graphs? This is the question. We deal with this with a graphon $U$, which produces disjoint cliques. Disjoint cliques are dense graphs. So how do we make sparse graphs? The trick is to take inverse line graphs of these disjoint cliques. Line graphs map edges to vertices and line graphs of stars are complete graphs.   -->

## The two graphons W and U

Let's plot the two graphons $W$ and $U$ first. 

```{r}
# create the dense graphon W(x,y) = exp(-(x+y)/40) where x and y ranges from 1 to 100
W <- create_exp_matrix(100, 40)
# plot this graphon
plot_graphon(W) +
 coord_fixed(ratio = 1) +
 ggtitle("Dense graphon W")  
```

The graphon $W$ above generates the dense part of the graph.  The graphon $U$, which we show below generates the sparse part of the graph. Note that graphon $U$ can be depicted by a sequence. In the following example, this sequence is *wts*. Essentially sampling from $U$ gives us a collection of star graphs where the number of node sequence in the stars is proportional to *wts*. Thus, the sparse part is a collection of stars.  

```{r}
# weights for the sparse part
seq <- 2:5
wts <- (1/1.2^seq) 
wts <- wts/sum(wts)
wts

U <- line_graphon(wts)
plot_graphon(U) + 
  coord_fixed(ratio = 1) +
 ggtitle("Line graphon U")  

```

## Generating the mixture graph in one step

Graphon $U$ is generated from weights *wts* in the above piece of code.  Given *wts* and $W$ we can generate a mixture graph using the function *sample_mixed_graph*. We need to specify the number of nodes in the dense part *nd* and the number of nodes in the sparse part *ns*. This generates a graph sampled from $W$ with *nd* nodes and a collection of stars with nodes proportional to *wts*. These two graphs are then joined. To join them we use the parameter *p*, which gives the proportion of the number of edges added in the joining process. That is, using  $p \times$ *number of edges in dense part*  we join nodes in the dense part with nodes in the sparse part. These edges are added randomly to nodes. 

<!-- Let's sample a mixture graph from a dense graphon $W$ and a disjoint clique graphon $U$, which acts as the sparse graphon. We call the sampled graph from $W$ as the dense part and the sampled graph from $U$ (equivalently from the sequence called a mass partition) the sparse part.  -->


```{r}

# single function to generate a graph mixture
gr1 <- sample_mixed_graph(W, wts, nd = 100, ns = 300, p = 0.5)
# plot(gr1, vertex.label = NA, vertex.size = 1, main = "(U,W) Graph mixture")
plot(gr1,
     edge.curved = 0.3,
     vertex.size = degree(gr1)*0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "(U,W) Graph mixture"
)
```


## Generating a dense and sparse graph and joining to get the mixture

Or we can generate the dense part and sparse part separately and join them.

```{r}
# sample dense part and plot it
grdense <- sample_graphon(W, n = 100)
plot(grdense,
     edge.curved = 0.3,
     vertex.size = degree(grdense) * 0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "Dense Part"
)

# sample sparse part and plot it
grsparse <- generate_star_union(wts, n = 300)
plot(grsparse,
     edge.curved = 0.3,
     vertex.size = degree(grsparse) * 0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "Sparse Part"
)

# join the two graphs and plot it
gr2 <- graph_join(grdense, grsparse, p = 0.5)
plot(gr2,
     edge.curved = 0.3,
     vertex.size = degree(gr2) * 0.1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue",
     main = "(U,W) Graph mixture"
)
```


# Estimating graphon U

If the graph sequence is sparse, then the high degrees of the graph are generated by graphon $U$. These are different from the low degrees. We can see it when we plot the sorted log degrees of the observed graph. 

## Observing the elbow point in unique log degrees

The graph below plots the sorted unique log degrees of the mixture graph. Notice the elbow point around indices 4 and 5. 

```{r}
degu <- sort(unique(degree(gr2)), decreasing = TRUE)
# we only take the top 1/2 of the unique degree values here to see the effect of the hub nodes clearly
degu <- degu[degu >= median(degu)]

df <- data.frame(
  index = 1:length(degu),
  log_degree = log(degu)
)

ggplot(df, aes(index, log_degree)) + 
 geom_point() + 
 xlab("Index") + 
 ylab("Log Degree")  


```

Let us check the degrees of the dense part and the sparse part and compare it with the degrees of the mixture graph

```{r}
deg_dense <- sort(unique(degree(grdense)), decreasing = TRUE)
deg_dense

deg_sparse <- sort(unique(degree(grsparse)), decreasing = TRUE)
deg_sparse

deg_mix <- sort(unique(degree(gr2)), decreasing = TRUE)
deg_mix
```

Of course the graph joining process increases the degrees of some nodes because nodes are joined by the additional edges.  

```{r}
which(deg_sparse > max(deg_dense))
```

## Detecting the elbow point and estimating $U$

From the mixture graph we can identify the sparse part. We do this by finding the elbow point of the log degree graph shown above. The function *extract_sparse* does the job. 

```{r}
out <- extract_sparse(gr2)
out$num_hubs

# Estimate of the mass-partition
out$phat

# Actual mass partition
wts
```

The estimated mass-partition is given in *out$phat* in the above code. The estimate is close to the actual mass partition given by *wts*.  

We can see the elbow point of the unique log degree graph with *autoplot*.

```{r}
autoplot(out)
```

Now that we have estimated the weights  (mass-partition) we can plot the estimated graphon $\hat{U}$.

```{r}
Uhat <- line_graphon(out$phat)
plot_graphon(Uhat) + 
  coord_fixed(ratio = 1) +
 ggtitle("Estimate of U")  

```


## References

---
title: "Introducing Graphettes"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introducing Graphettes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib  
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(igraph)
library(graphonmix)
library(ggplot2)
```


Graphettes are a graph generation methodology proposed in [@wijesinghe2026flowette]. A graphette has 3 components: a graphon $W$, a sequence $\{\rho_n\}_n$ and a graph edit function $f$. We can generate dense or sparse graphs from a graphette. On the sparse graph front, we can generate diverse sparse graphs, such as ring-added graphs that have bounded degree or graphs with large hubs like this social networks. We can generate trees as well. Let's look at some examples.

# Dense graphs from graphons and graphettes
Let's start with a stochastic block model graphon. 

```{r}
mat <- matrix(c(0.9, 0.01, 0.02,
                0.01, 0.8, 0.03,
                0.02, 0.03, 0.7), nrow = 3, byrow = TRUE)
W <- sbm_graphon(mat, 100)
plot_graphon(W) +  coord_fixed(ratio = 1) 
```

## Using the graphon

Without using graphettes we can sample from this graphon as follows:

```{r}
gr <- sample_graphon(W, 100)
plot(gr, vertex.size = 3, vertex.label = NA, main = "Graphon sampling")
```

## Using the graphette

To generate dense graphs from the graphette we set $\rho_n = 1$ for all $n$ and the graph edit function $f$ to the identity function. In the function call *sample_graphette* we set 
$\rho_n$ = NULL and graph_edit_function = NULL. That way, we get are actually sampling from the graphon $W$ without sparsifying it.  


```{r}
gr1 <- sample_graphette(W, n= 100)
plot(gr1, vertex.size = 3, vertex.label = NA, main = "Graphette sampling")


```

As you can see, in this instance sampling from the graphon or the graphette gives similar graphs. 

# Sparse graphs using $\rho_n$
Consider the same SBM graphon. We can sparsify it by considering $W_n = \rho_n W$ where $\rho_n \to 0$. And then we sample a graph $G_n$ with $n$ nodes from $W_n$. As $n \to \infty$ the edge density goes to zero making this graph sequence sparse. Let's see how this works. 

## Using the standard graphon and $\rho_n$ separately

```{r}

rho_n <- function(n) exp( -((n-100)/50 ))


W250 <- rho_n(250)*W
W300 <- rho_n(300)*W

gr1 <- sample_graphon(W, 100)
gr2 <- sample_graphon(W250, 250)
gr3 <- sample_graphon(W300, 300)


g1 <- plot_graphon(W) +  coord_fixed(ratio = 1) 
g2 <- plot_graphon(W250) +  coord_fixed(ratio = 1) 
g3 <- plot_graphon(W300) + coord_fixed(ratio = 1) 

gridExtra::grid.arrange(g1, g2, g3, nrow = 1)
```

The three graphons are shown above and we see the colours get greyed out as $\rho_n$ goes to zero. Let's plot the graphs sampled from the two graphons. 

```{r}
par(mfrow = c(1,3))
plot(gr1, vertex.size = 3, vertex.label = NA, main = "From W")
plot(gr2, vertex.size = 3, vertex.label = NA, main = "From W250")
plot(gr3, vertex.size = 3, vertex.label = NA, main = "From W300")
par(mfrow = c(1,1)) 

```

We see the graphs getting sparser. 

## Using the function *sample_sparse_graphon*

The function *sample_sparse_graphon* has a function parameter that can be used for $\rho_n$. 


```{r}
gr4 <- sample_sparse_graphon(W, rho_n, n = 250)
gr5 <- sample_sparse_graphon(W, rho_n, n = 300)

par(mfrow = c(1,2))
plot(gr4, vertex.size = 3, vertex.label = NA, main = "n = 250")
plot(gr5, vertex.size = 3, vertex.label = NA, main = "n = 300")
par(mfrow = c(1,1))
```
## Using the graphette

We can use the *sample_graphette* to get the same functionality by setting *graph_edit_f = $NULL. 

```{r}
gr6 <- sample_graphette(W, rho_n, n = 250)
gr7 <- sample_graphette(W, rho_n, n = 300)


par(mfrow = c(1,2))
plot(gr6, vertex.size = 3, vertex.label = NA, main = "n = 250")
plot(gr7, vertex.size = 3, vertex.label = NA, main = "n = 300")
par(mfrow = c(1,1))

```

# Sparse graphs using $\rho_n$ and the graph edit function
We have the functionality to add stars, add rings and remove cycles using the graph edit function. 

## Adding stars
The functions *star_f1*, *star_f2*, *star_f3*, *star_f4* and *star_f5* can be used to add stars. While there are slight differences, all of these functions use a Poisson process to add additional connections to each node. The parameter $t_or_p$ controls the intensity of the Poisson process. 

```{r}
gr8 <- sample_graphette(W, rho_n, "star_f1", n = 250, t_or_p = 2)
gr9 <- sample_graphette(W, rho_n, "star_f1", n = 300, t_or_p = 2)

par(mfrow = c(1,2))
plot(gr8, vertex.size = 3, vertex.label = NA, main = "n = 250")
plot(gr9, vertex.size = 3, vertex.label = NA, main = "n = 300")
par(mfrow = c(1,1))
```
In the above example $\rho_n$ goes to 0 very fast. Let us use something a bit different

```{r}
rho2 <- function(n)10/n
gr10 <- sample_graphette(W, rho2, "star_f1", n = 250, t_or_p = 2)
gr11 <- sample_graphette(W, rho2, "star_f1", n = 300, t_or_p = 2)

par(mfrow = c(1,2))
plot(gr10, vertex.size = 3, vertex.label = NA, main = "n = 250")
plot(gr11, vertex.size = 3, vertex.label = NA, main = "n = 300")
par(mfrow = c(1,1))
```
## Adding rings

To add rings to the graph, use *add_rings* as the graph edit function. The parameter *t_or_p* gives the proportion of nodes to which rings are added. The default ring size is 5 or 6, but this can be changed. 

```{r}
gr <- sample_graphette(W, rho_n, "add_rings", n = 100, t_or_p = 0.5, ring_sizes = c(10:15))

plot(gr, vertex.size = 3, vertex.label = NA)

```

## Removing cycles

To remove cycles from a graph, use *remove_cycles* as the graph edit function. 

```{r}
gr <- sample_graphette(W, rho_n, "remove_cycles", n = 200)

plot(gr, vertex.size = 3, vertex.label = NA)

```

## References

---
title: "Sparse graphs from line graphons"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Sparse graphs from line graphons}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(graphonmix)
library(ggplot2)
library(igraph)
requireNamespace("gridExtra", quietly = TRUE)
```

# Line graphs 

Suppose $G$ is a graph. Then the line graph of $G$, commonly denoted by $L(G)$ maps edges of $G$ to vertices of $L(G)$ and two vertices in $L(G)$ are connected if the original edges in $G$ share a common vertex.  Here's an example. 

```{r}
g1 <- make_star(4,mode = "undirected")
g1 <- add_vertices(g1, 1)
g1 <- add_edges(g1,c(4,5))

lg1 <- make_line_graph(g1)

oldpar <- par(mfrow = c(1,2))
plot(g1, vertex.label = NA, edge.label = c(1,2,3,4), main = "Graph G")
plot(lg1, vertex.label = c(1,2,3,4), vertex.size = 20, main = "Line graph L(G)")
par(oldpar)
```

## Line graphs of star graphs

The key is that line graphs of stars are complete graphs. In the example below each edge of graph $G$ is a vertex in $L(G)$. Two vertices in $L(G)$ are connected if the corresponding edges in $G$ share a vertex. 

```{r}
st <- make_star(10, mode = "undirected")
lgr_st <- make_line_graph(st)

oldpar <- par(mfrow = c(1,2))
plot(st, vertex.label = NA, main = "Star graph G")
plot(lgr_st, vertex.label = NA,  main = "Line graph L(G) of a star")
par(oldpar)
```

# Sparse graphs

A sequence of graphs is sparse if the edges grow subquadratically with the nodes. Star graphs are the ultimate sparse graphs, because a star of $n$ nodes has $n-1$ edges. A graph sequence is dense if the edges grow quadratically with the nodes. Erdos-Renyi graphs $G(n,p)$ with a fixed edge probability $p$ are dense. This can be seen from the edge density. Let's generate a sequence of star graphs and a sequence of Erdos-Renyi graphs and see this in action. 

```{r}
stardensity <- gnpdensity <- rep(0, 20)
 
for(i in 1:20){
  n <- i*100
  gr1 <- sample_gnp(n, p = 0.1)
  gnpdensity[i] <- edge_density(gr1)
  gr2 <- make_star(n, mode = "undirected")
  stardensity[i] <- edge_density(gr2)
}
gnpdensity
stardensity
```

We see that the density of stars goes to zero whereas the density of GNP graphs hover around 0.1, because we have used the edge probability as 0.1.  


# Graphons

Graphons are graph limits. Suppose we have a graph $G$. The empirical graphon of $G$ is when we scale the adjacency matrix to the unit square and color the small squares that correspond to ones in the adjacency matrix in black. Here we plot the empirical graphon of a star graph with 10 nodes. 

```{r}
gr <- make_star(10, mode = "undirected")
emp <- empirical_graphon(gr)
plot_graphon(emp) + coord_fixed()
```

## Graphons of sparse graphs

The problem is graphons of sparse graphs are zero. Graphons are generally denoted by the letter $W$. And if the graphs are sparse then $W = 0$. That is, the empirical graphons converge to zero. Here, we're a bit handwavy about convergence (actually, the convergence is with respect to the cut metric, but let's let's not discuss that).

```{r}
gr <- make_star(20, mode = "undirected")
emp <- empirical_graphon(gr)
g1 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 10')

gr <- make_star(100, mode = "undirected") 
emp <- empirical_graphon(gr)
g2 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 100')

gr <- make_star(200, mode = "undirected")
emp <- empirical_graphon(gr)
g3 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 200')


gridExtra::grid.arrange(g1, g2, g3, nrow = 1)

```

As $n \to \infty$, there are less black parts in the empirical graphon and it converges to zero. When the graphon is zero, we cannot sample from it; it is not useful anymore. So what do we do?

# Graphons of line graphs

For a subset of sparse graphs, such as the star graphs, their line graphs are dense. We discuss this in detail in [@SKCS2024graphons]. The main example is star graphs. 

```{r}
gr1 <- make_star(10, mode = "undirected")
gr <- gr1 %du% gr1
lgr <- make_line_graph(gr)
emp <- empirical_graphon(lgr)
g1 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 20')


gr1 <- make_star(50, mode = "undirected")
gr <- gr1 %du% gr1
lgr <- make_line_graph(gr)
emp <- empirical_graphon(lgr)
g2 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 100')


gr1 <- make_star(100, mode = "undirected")
gr <- gr1 %du% gr1
lgr <- make_line_graph(gr)
emp <- empirical_graphon(lgr)
g3 <- plot_graphon(emp) + coord_fixed() + ggtitle('n = 200')


gridExtra::grid.arrange(g1, g2, g3, nrow = 1)


```

For the above example of two stars, the line graphs are dense and converge to a non-zero graphon. 


## Janson's theorem

Janson [@janson2016graph] proved that all line graph limits are disjoint clique graphs. This is a very important theorem for us because it tells us what the structure of line graph limits are. Janson explains that line graph limits can be written as a sequence of numbers $(p_1, p_2, \ldots)$ such that $\sum_i p_i \leq 1$ and $p_i \geq 0$. The $p_i$ gives the width of each black box in the graphon.   


## Generating stars as inverse line graphs
From a line graph limit we can generate a disjoint union of clique graphs like this. 

```{r}
wts <- c(0.5, 0.3, 0.2)
U <-  line_graphon(wts)
gr <- sample_graphon(U, n = 100)
plot(gr,
     vertex.size = 1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue"
)
```

The inverse line graphs of disjoint clique graphs are star graphs. And star graphs are sparse. We can generate star graphs using the weights or the partition $(p_1, p_2, \ldots)$.

```{r}
grsp <- generate_star_union(wts, n = 100)
plot(grsp,
     edge.curved = 0.3,
     vertex.size = 1,
     edge.color = "lightgray",     # Light colored edges
     vertex.label = NA,
     vertex.color = "lightblue"
)
```

This is how the sparse part of the $(U,W)$-mixture graph is generated. The vignette at  *Articles/Getting Started* explains the mixture procedure. 

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plot_graphon}
\alias{plot_graphon}
\title{Plots graphon}
\usage{
plot_graphon(W, cols = c("white", "black"))
}
\arguments{
\item{W}{A graphon given by an nxn matrix}

\item{cols}{Colors, by default white and black}
}
\value{
A ggplot object.
}
\description{
Plots graphon
}
\examples{
W <- create_exp_matrix(100, 100)
plot_graphon(W)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphon_generation.R
\name{scale_graphon}
\alias{scale_graphon}
\title{Scales a graphon to an nxn matrix}
\usage{
scale_graphon(W, n)
}
\arguments{
\item{W}{A graphon given as a symmetric square matrix}

\item{n}{The dimension of the output matrix}
}
\value{
Scaled nxn graphon
}
\description{
Scales a graphon to an nxn matrix suitable for large adjacency matrices
}
\examples{
library(igraph)
gr <- sample_gnp(1000, p=0.2)
adj <- as_adjacency_matrix(gr)
W <- scale_graphon(adj, 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphon_generation.R
\name{generate_star_union}
\alias{generate_star_union}
\title{Generates a sparse graph of star graphs}
\usage{
generate_star_union(wts, n)
}
\arguments{
\item{wts}{The proportion of the hub degrees}

\item{n}{The number of nodes in the new graph}
}
\value{
A disjoint union of star graphs
}
\description{
Generates a union of star graphs given the weights
}
\examples{
library(igraph)
wts <- c(0.5, 0.3, 0.2)
gr <- generate_star_union(wts, n = 100)
gr

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_sparse.R
\name{extract_sparse}
\alias{extract_sparse}
\title{Extracts the sparse part from a (U,W) graphon mixture}
\usage{
extract_sparse(gr)
}
\arguments{
\item{gr}{The input graph}
}
\value{
a list with the following components:
\item{\code{num_hubs}}{The number of hubs in the sparse component.}
\item{\code{phat}}{The probability vector of the sparse component. This is also known as the mass partition.}
\item{\code{segment_sizes}}{The sizes of the two line segments.}
\item{\code{line_equations}}{The two equations of the lines.}
\item{\code{cutoff}}{The best cut off for the two lines.}
\item{\code{models}}{The models of the fitted lines. }
\item{\code{mse}}{The mean squared error}
\item{\code{data}}{The degree data.}
}
\description{
This function extracts the sparse component from a (U,W) mixture graph by
fitting two lines to the unique sorted log degree values. The first line gives
the number of hubs in the sparse part and the remaining line fits the degrees
of the dense part.
}
\examples{
library(igraph)
gr <- sample_pa(10000, power = 1.2, directed = FALSE)
sparse <- extract_sparse(gr)
sparse$phat

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphettes.R
\name{sample_sparse_graphon}
\alias{sample_sparse_graphon}
\title{Samples a graph from a sparsified graphon}
\usage{
sample_sparse_graphon(W, rho_n = NULL, n)
}
\arguments{
\item{W}{The graphon. This is a symmetric matrix with values in [0,1]}

\item{rho_n}{A function. Default is 1.}

\item{n}{The number of nodes in the graph to sample}
}
\value{
gr A graph sampled from rho_n W.
}
\description{
Samples a graph from a sparsified graphon given by a_n W, where W is the graphon and a_n is generally a sequence converging to zero.
}
\examples{
library(igraph)
f1 <- function(n) 10/n
W <- matrix(0.2, nrow = 100, ncol = 100)
gr <- sample_sparse_graphon(W, f1, n = 100)
gr

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphon_generation.R
\name{sample_graphon}
\alias{sample_graphon}
\title{Generates a graph given a graphon}
\usage{
sample_graphon(W, n)
}
\arguments{
\item{W}{A graphon given by a matrix}

\item{n}{The number of nodes of the sampled graph}
}
\value{
A graph sampled from the graphon W with n nodes
}
\description{
Generates a graph given a dense graphon W.
}
\examples{
library(igraph)
W <- matrix(0.2, nrow = 100, ncol = 100)
gr <- sample_graphon(W, n= 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphon_generation.R
\name{sample_mixed_graph}
\alias{sample_mixed_graph}
\title{Generate a (U,W) mixture graph}
\usage{
sample_mixed_graph(W, wts, nd, ns, p = 0.5, option = 2)
}
\arguments{
\item{W}{The dense graphon. This is a symmetric matrix with values in [0,1]}

\item{wts}{The degree proportions of the hub degrees. Need to add up to 1. This
is the mass partition corresponding to the line graph graphon U.}

\item{nd}{The number of nodes in the dense part of the graph}

\item{ns}{The number of nodes in the sparse part of the graph}

\item{p}{The nodes to be added as a proportion of the edges in the dense part}

\item{option}{Graph joining option. If \code{option == 1} then a disjoit union is
considered. If \code{option == 2} the two graphs are joined randomly with the
number of edges specified by \code{p}.}
}
\value{
A graph sampled from the (U,W) mixture.
}
\description{
Generate a (U,W) mixture graph from a dense graphon W and a mass partition
corresponding to a line graph graphon U.
}
\examples{
library(igraph)
W <- matrix(0.1, nrow = 100, ncol = 100)
wts <- c(0.5, 0.3, 0.2)
ns <- 200
nd <- 100
p <- 0.5
gr <- sample_mixed_graph(W, wts, nd, ns, p, option = 2)
gr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphonmix-package.R
\docType{package}
\name{graphonmix-package}
\alias{graphonmix}
\alias{graphonmix-package}
\title{graphonmix: Generates Dense and Sparse Graphs using Graphon Extensions}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Generates dense or sparse graphs using graphon mixtures and graphettes. Graphon mixtures uses two graphons U and W to generate graphs. Sparse graphs are generated in this case using the inverse line graph (root) operation. Graphettes have 3 components, the graphon W, a sparsifying sequence and a graph edit function. Both techniques can generate dense or sparse graphs. Kandanaarachchi and Ong (2026) \doi{10.48550/arXiv.2505.13864}, Wijesinghe et al (2026) \doi{10.48550/arXiv.2602.23566}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://sevvandi.github.io/graphonmix/}
}

}
\author{
\strong{Maintainer}: Sevvandi Kandanaarachchi \email{sevvandik@gmail.com} (\href{https://orcid.org/0000-0002-0337-0395}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphon_generation.R
\name{empirical_graphon}
\alias{empirical_graphon}
\title{Computes empirical graphon from graph}
\usage{
empirical_graphon(gr, n = NULL)
}
\arguments{
\item{gr}{A graph}

\item{n}{Dimension of the graphon matrix}
}
\value{
The empirical graphon
}
\description{
Computes empirical graphon given a graph
}
\examples{
library(igraph)
gr <- sample_gnp(1000, p=0.2)
emp <-  empirical_graphon(gr, n = 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphon_generation.R
\name{graph_join}
\alias{graph_join}
\title{Joins two graphs}
\usage{
graph_join(gr1, gr2, p = 0.5, option = 2)
}
\arguments{
\item{gr1}{The first graph to join}

\item{gr2}{The second graph to join}

\item{p}{The proportion of edges in \code{gr1} to be added as part of the joining}

\item{option}{Two options. 1 does the disjoint union, 2 does the random edges union.}
}
\value{
The joined graph
}
\description{
Joins two graphs randomly connecting vertices
}
\examples{
W <- create_exp_matrix(100, 100)
# create the sparse part - a disjoint set of stars
wts <- c(0.5, 0.3, 0.2)
grdense <- sample_graphon(W, 100)
grsparse <- generate_star_union(wts, 200)
gr <- graph_join(grdense, grsparse, opt = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot_separate_dense_sparse.R
\name{autoplot.separate_dense_and_sparse}
\alias{autoplot.separate_dense_and_sparse}
\title{Plots the output of separate_dense_and_sparse.}
\usage{
\method{autoplot}{separate_dense_and_sparse}(object, ...)
}
\arguments{
\item{object}{The output of the function 'separate_dense_and_sparse.'}

\item{...}{Other arguments currently ignored}
}
\value{
A ggplot object.
}
\description{
Plots the three lines fitted to the unique, sorted log degrees of the graph.
}
\examples{
library(igraph)
W <- matrix(0.1, nrow = 100, ncol = 100)
wts <- c(0.5, 0.3, 0.2)
ns <- 200
nd <- 100
p <- 0.5
gr <- sample_mixed_graph(W, wts, nd, ns, p = 0.1, option = 2)
out <- separate_dense_and_sparse(gr)
separate <- separate_dense_and_sparse(gr)
autoplot(separate)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_separation.R
\name{separate_dense_and_sparse}
\alias{separate_dense_and_sparse}
\title{Separates the dense and sparse part from a (U,W) graphon mixture}
\usage{
separate_dense_and_sparse(grmix)
}
\arguments{
\item{grmix}{The input graph}
}
\value{
a list with the following components:
\item{\code{gr_dense}}{The dense component.}
\item{\code{gr_sparse}}{The sparse component.}
\item{\code{data}}{The original graph.}
}
\description{
This function breaks a (U,W) mixture graph into a dense and sparse component.
}
\examples{
library(igraph)
W <- matrix(0.1, nrow = 100, ncol = 100)
wts <- c(0.5, 0.3, 0.2)
ns <- 200
nd <- 100
p <- 0.5
gr <- sample_mixed_graph(W, wts, nd, ns, p = 0.1, option = 2)
out <- separate_dense_and_sparse(gr)
out

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphon_generation.R
\name{line_graphon}
\alias{line_graphon}
\title{Creates a line graphon from a sequence of probabilities}
\usage{
line_graphon(probs)
}
\arguments{
\item{probs}{The list of probabilities starting from the largest.}
}
\value{
The line graphon
}
\description{
Creates a line graphon, which is a disjoint clique graphon from a
sequence of probability values
}
\examples{
library(ggplot2)
wts <- c(0.5, 0.3, 0.2)
U <- line_graphon(wts)
plot_graphon(U)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphettes.R
\name{sample_graphette}
\alias{sample_graphette}
\title{Samples a graph from a graphette}
\usage{
sample_graphette(
  W,
  rho_n = NULL,
  graph_edit_f = NULL,
  n = 100,
  t_or_p = 0.5,
  ring_sizes = c(5, 6)
)
}
\arguments{
\item{W}{The graphon. This is a symmetric matrix with values in [0,1]}

\item{rho_n}{A function. Default is 1.}

\item{graph_edit_f}{The graph edit function. For star functions it can be
one of \code{'star_f1'}, \code{'star_f2'}, \code{'star_f3'}, \code{'star_f4'} or \code{'star_f5'}. To add rings, it needs to be \code{'add_rings'}. The function \code{'remove_cycles'} removes cycles.}

\item{n}{The number of nodes in the graph to sample}

\item{t_or_p}{The parameter for star function or the ring function. For the
ring function it needs to be between 0 and 1.}

\item{ring_sizes}{The size of rings to add if the graph edit function is the ring function. default values set to \code{c(5,6)}.}
}
\value{
gr A graph sampled from rho_n W.
}
\description{
Samples a graph from a graphette given by (W, a_n, f), where W is the graphon, a_n is generally a sequence converging to zero and f is a graph edit funcion.
}
\examples{
library(igraph)
# Example 1
f1 <- function(n) 10/n
W <- matrix(0.2, nrow = 100, ncol = 100)
gr <- sample_graphette(W, rho_n = f1, graph_edit_f = 'add_rings', n = 100, t_or_p = 0.5)
gr

# Example 2
gr <- sample_graphette(W, rho_n = f1, graph_edit_f = 'star_f1', n = 100, t_or_p = 3)
gr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ring_graphon}
\alias{ring_graphon}
\title{Creates a ring graphon}
\usage{
ring_graphon(n, alpha = 0.1)
}
\arguments{
\item{n}{The dimension of the matrix}

\item{alpha}{A scaling factor used in the graphon. Default set to 0.1}
}
\value{
An nxn matrix
}
\description{
Creates an graphon which can generate ring graphs.
}
\examples{
# example code
W <- ring_graphon(100)
plot_graphon(W)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{create_exp_matrix}
\alias{create_exp_matrix}
\title{Creates an exponential graphon}
\usage{
create_exp_matrix(nrow, scalar)
}
\arguments{
\item{nrow}{The dimension of the matrix}

\item{scalar}{The scalar in exp(-(i+j)/scalar)}
}
\value{
An nxn matrix
}
\description{
Creates an nxn matrix where the (i,j)th entry is exp(-(i+j)/scalar)
}
\examples{
W <- create_exp_matrix(100, 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sbm_graphon}
\alias{sbm_graphon}
\title{Creates a Stochastic Block Model graphon}
\usage{
sbm_graphon(mat, n)
}
\arguments{
\item{mat}{The matrix representing the SBM}

\item{n}{The dimension of the matrix}
}
\value{
An nxn matrix
}
\description{
Creates an graphon representing a Stochastic Block Model (SBM).
}
\examples{
# example code
mat <- matrix(c(0.9, 0.01, 0.02,
0.01, 0.8, 0.03,
0.02, 0.03, 0.7), nrow = 3, byrow = TRUE)
W <- sbm_graphon(mat, 100)
plot_graphon(W)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_hubs.R
\name{predict_hubs}
\alias{predict_hubs}
\title{Predicts the degree of hubs of an new graph}
\usage{
predict_hubs(gr, n, k = NULL)
}
\arguments{
\item{gr}{The input graph}

\item{n}{The number of nodes in the new graph}

\item{k}{The number of hubs. Default is \code{NULL}}
}
\value{
A vector of hub degrees
}
\description{
Predicts the degree of hubs of an unseen graph given a graph generated
from the same process.
}
\examples{
library(igraph)
gr <- sample_pa(10000, power = 1.2, directed = FALSE)
predict_hubs(gr, n = 11000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphettes.R
\name{star_f3}
\alias{star_f3}
\title{Adds stars to nodes}
\usage{
star_f3(xvals, t)
}
\arguments{
\item{xvals}{Values between 0 and 1 from the graphon}

\item{t}{A parameter indicating the strength of hubs}
}
\value{
a vector of values
}
\description{
Adds stars to nodes
}
\examples{
# example code
st <- star_f3(runif(10), 3)
st
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot_extract_sparse.R
\name{autoplot.extract_sparse}
\alias{autoplot.extract_sparse}
\title{Plots the output of extract_sparse function.}
\usage{
\method{autoplot}{extract_sparse}(object, ...)
}
\arguments{
\item{object}{The output of the function 'extract_sparse'}

\item{...}{Other arguments currently ignored}
}
\value{
A ggplot object.
}
\description{
Plots the two lines fitted to the unique, sorted log degrees of the graph.
}
\examples{
library(igraph)
gr <- sample_pa(10000, power = 1.2, directed = FALSE)
sparse <- extract_sparse(gr)
autoplot(sparse)

}
